/**
 * app.js
 * 
 * Main application file for Fylgja Slack bot
 * Initializes the Slack Bolt app with custom ExpressReceiver Registers command handlers
 * Now supports the universal /fylgja command
 */
const { App, ExpressReceiver } = require('@slack/bolt');
const fs = require('fs');
const logger = require('./utils/logger');
const { SIGMA_CLI_PATH, SIGMA_CLI_CONFIG, SLACK_CONFIG } = require('./config/appConfig');

const { getFileName } = require('./utils/file_utils');
const FILE_NAME = getFileName(__filename);

// Import the unified fylgja command handler
const fylgjaCommandHandler = require('./handlers/fylgja_command_handler');

const sigmaDetailsHandler = require('./handlers/sigma/sigma_details_handler');
const sigmaSearchHandler = require('./handlers/sigma/sigma_search_handler');
const sigmaCreateHandler = require('./handlers/sigma/sigma_create_handler');
// Import the action registry
const sigmaActionRegistry = require('./handlers/sigma/actions/sigma_action_registry');

// Verify sigma-cli is installed
if (!fs.existsSync(SIGMA_CLI_PATH)) {
  logger.error(`Error: Sigma CLI not found at specified path: ${SIGMA_CLI_PATH}`);
  process.exit(1);
}

// Log the loaded configuration
logger.info(`Loaded sigma-cli configuration: ${JSON.stringify(SIGMA_CLI_CONFIG)}`);

/**
 * Create a custom ExpressReceiver for more control over the HTTP server
 */
const expressReceiver = new ExpressReceiver({
  signingSecret: SLACK_CONFIG.signingSecret,
  processBeforeResponse: true
});

/**
 * Initialize the Slack app with the custom receiver
 */
const app = new App({
  token: SLACK_CONFIG.botToken,
  receiver: expressReceiver
});

// Register the unified fylgja command handler
logger.info('Registering unified fylgja command handler');
app.command('/fylgja', async ({ command, ack, respond }) => {
  try {
    await ack();
    logger.info(`Received fylgja command: ${command.text}`);
    await fylgjaCommandHandler.handleCommand(command, respond);
  } catch (error) {
    logger.error(`Error handling fylgja command: ${error.message}`);
    logger.debug(`Error stack: ${error.stack}`);
    await respond({
      text: `An error occurred: ${error.message}`,
      response_type: 'ephemeral'
    });
  }
});


// Register sigma command handlers directly
app.command('/sigma-create', async ({ command, ack, respond }) => {
  try {
    await ack();
    logger.info(`Received sigma-create command: ${command.text}`);
    await sigmaCreateHandler.handleCommand(command, respond);
  } catch (error) {
    logger.error(`Error handling sigma-create command: ${error.message}`);
    logger.debug(`Error stack: ${error.stack}`);
    await respond({
      text: `An error occurred: ${error.message}`,
      response_type: 'ephemeral'
    });
  }
});

app.command('/sigma-details', async ({ command, ack, respond }) => {
  try {
    await ack();
    logger.info(`Received sigma-details command: ${command.text}`);
    await sigmaDetailsHandler.handleCommand(command, respond);
  } catch (error) {
    logger.error(`Error handling sigma-details command: ${error.message}`);
    logger.debug(`Error stack: ${error.stack}`);
    await respond({
      text: `An error occurred: ${error.message}`,
      response_type: 'ephemeral'
    });
  }
});

app.command('/sigma-search', async ({ command, ack, respond }) => {
  try {
    await ack();
    logger.info(`Received sigma-search command: ${command.text}`);
    await sigmaSearchHandler.handleCommand(command, respond);
  } catch (error) {
    logger.error(`Error handling sigma-search command: ${error.message}`);
    logger.debug(`Error stack: ${error.stack}`);
    await respond({
      text: `An error occurred: ${error.message}`,
      response_type: 'ephemeral'
    });
  }
});

app.command('/sigma-stats', async ({ command, ack, respond }) => {
  try {
    await ack();
    logger.info(`Received sigma-stats command`);
    const sigmaStatsHandler = require('./handlers/sigma/sigma_stats_handler');
    await sigmaStatsHandler.handleCommand(command, respond);
  } catch (error) {
    logger.error(`Error handling sigma-stats command: ${error.message}`);
    logger.debug(`Error stack: ${error.stack}`);
    await respond({
      text: `An error occurred: ${error.message}`,
      response_type: 'ephemeral'
    });
  }
});

// Register all button action handlers from the modular registry
sigmaActionRegistry.registerActionHandlers(app);

/**
 * Listen for any message in DMs
 * This allows users to interact with the bot directly
 */
app.message(async ({ message, say }) => {
  // Only respond to DMs (message.channel starts with 'D')
  if (message.channel.startsWith('D')) {
    logger.info(`DM received from user ${message.user}`);
    logger.debug(`DM content: ${message.text}`);
    
    // For now, we're just logging DMs but not responding
    // Uncomment below to enable responses to DMs
    // await say(`I received your message: "${message.text}"`);
  }
});

/**
 * Export the configured app for use in the main server file
 */
module.exports = app;