/**
 * sigma_search_handler.js
 * 
 * Handles Sigma rule search requests from Slack commands
 */

const { searchSigmaRules, searchSigmaRulesComplex, searchAndConvertRules } = require('../../services/sigma/sigma_search_service');
const logger = require('../../utils/logger');
const { handleError } = require('../../utils/error_handler');
const { getSearchResultBlocks } = require('../../blocks/sigma/sigma_search_results_block');
const { getFileName } = require('../../utils/file_utils');
const FILE_NAME = getFileName(__filename);

const MAX_RESULTS_PER_PAGE = 10;
const MAX_RESULTS_THRESHOLD = 99;

/**
 * Handle the sigma-search command for Sigma rules
 * Searches for rules based on keywords and displays results with pagination
 * 
 * @param {Object} command - The Slack command object
 * @param {Function} respond - Function to send response back to Slack
 */
const handleCommand = async (command, respond) => {
  try {
    logger.debug(`${FILE_NAME}: Processing sigma-search command: ${JSON.stringify(command.text)}`);
    
    if (!command || !command.text) {
      logger.warn(`${FILE_NAME}: Empty command received for sigma-search`);
      await respond('Invalid command. Usage: /sigma-search [keyword]');
      return;
    }
    
    // Extract search keyword and check for pagination parameters
    let keyword = command.text.trim();
    let page = 1;
    let pageSize = MAX_RESULTS_PER_PAGE;
    
    // Check for pagination format: keyword page=X
    const pagingMatch = keyword.match(/(.+)\s+page=(\d+)$/i);
    if (pagingMatch) {
      keyword = pagingMatch[1].trim();
      page = parseInt(pagingMatch[2], 10) || 1;
      logger.debug(`${FILE_NAME}: Detected pagination request: "${keyword}" page ${page}`);
    }
    
    // Check for page size format: keyword limit=X
    const limitMatch = keyword.match(/(.+)\s+limit=(\d+)$/i);
    if (limitMatch) {
      keyword = limitMatch[1].trim();
      pageSize = parseInt(limitMatch[2], 10) || MAX_RESULTS_PER_PAGE;
      // Ensure the page size is within reasonable limits
      pageSize = Math.min(Math.max(pageSize, 1), 100);
      logger.debug(`${FILE_NAME}: Detected page size request: "${keyword}" limit ${pageSize}`);
    }
    
    if (!keyword) {
      logger.warn(`${FILE_NAME}: Missing keyword in sigma-search command`);
      await respond('Invalid command: missing keyword. Usage: /sigma-search [keyword]');
      return;
    }
    
    logger.info(`${FILE_NAME}: Searching for rules with keyword: ${keyword} (page ${page}, size ${pageSize})`);
    logger.debug(`${FILE_NAME}: Search keyword length: ${keyword.length}`);
    
    await respond({
      text: 'Searching for rules... This may take a moment.',
      response_type: 'ephemeral'
    });
    
    // Search for rules using the service function with pagination
    const searchResult = await searchAndConvertRules(keyword, page, pageSize);
    logger.debug(`${FILE_NAME}: Search result status: ${searchResult.success}`);
    logger.debug(`${FILE_NAME}: Found ${searchResult.results?.length || 0} results out of ${searchResult.pagination?.totalResults || 0} total matches`);
    logger.debug(`${FILE_NAME}: About to generate blocks for search results`);
    
    if (!searchResult.success) {
      logger.error(`${FILE_NAME}: Search failed: ${searchResult.message}`);
      await respond({
        text: `Search failed: ${searchResult.message}`,
        response_type: 'ephemeral'
      });
      return;
    }
    
    // Get total count for validation
    const totalCount = searchResult.pagination?.totalResults || 0;
    
    // Check if search returned too many results
    if (totalCount > MAX_RESULTS_THRESHOLD) {
      logger.warn(`${FILE_NAME}: Search for "${keyword}" returned too many results (${totalCount}), displaying first page with warning`);
      // Continue processing but add a notification
      searchResult.tooManyResults = true;
    }
    
    if (!searchResult.results || searchResult.results.length === 0) {
      if (totalCount > 0) {
        logger.warn(`${FILE_NAME}: No rules found on page ${page} for "${keyword}", but ${totalCount} total matches exist`);
        await respond({
          text: `No rules found on page ${page} for "${keyword}". Try a different page or refine your search.`,
          response_type: 'ephemeral'
        });
      } else {
        logger.warn(`${FILE_NAME}: No rules found matching "${keyword}"`);
        await respond({
          text: `No rules found matching "${keyword}"`,
          response_type: 'ephemeral'
        });
      }
      return;
    }
    
    // Generate blocks with pagination support
    let blocks;
    try {
      logger.debug(`${FILE_NAME}: Calling getSearchResultBlocks with ${searchResult.results.length} results`);
      // If we have too many results, add a warning block at the beginning
      if (searchResult.tooManyResults) {
        blocks = getSearchResultBlocks(keyword, searchResult.results, searchResult.pagination);
        // Insert warning at the beginning of blocks (after the header)
        blocks.splice(1, 0, {
          "type": "section",
          "text": {
            "type": "mrkdwn",
            "text": `:warning: Your search for "${keyword}" returned ${totalCount} results, which is a lot. Displaying the first page. Consider using a more specific keyword for narrower results.`
          }
        });
      } else {
        blocks = getSearchResultBlocks(keyword, searchResult.results, searchResult.pagination);
      }
      logger.debug(`${FILE_NAME}: Successfully generated ${blocks?.length || 0} blocks`);
    } catch (blockError) {
      // Use error handler for block generation errors
      await handleError(blockError, `${FILE_NAME}: Block generation`, respond, {
        responseType: 'in_channel',
        customMessage: `Found ${searchResult.results.length} of ${totalCount} rules matching "${keyword}" (page ${page} of ${searchResult.pagination?.totalPages || 1}). Use /sigma-details [id] to view details.`
      });
      return;
    }
    
    // Add debug log before sending response
    logger.debug(`${FILE_NAME}: About to send response with ${blocks?.length || 0} blocks`);
    
    // Determine if this should be visible to everyone or just the user
    const isEphemeral = totalCount > 20;
    
    // Respond with the search results
    await respond({
      blocks: blocks,
      response_type: isEphemeral ? 'ephemeral' : 'in_channel'
    });
    
    // Add debug log after sending response
    logger.debug(`${FILE_NAME}: Response sent successfully`);
  } catch (error) {
    // Use error handler for unexpected errors
    await handleError(error, `${FILE_NAME}: Search command handler`, respond, {
      responseType: 'ephemeral'
    });
  }
};

/**
 * Handle the complex search command for Sigma rules
 * Processes advanced search queries with multiple conditions
 * 
 * @param {Object} command - The Slack command object
 * @param {Function} respond - Function to send response back to Slack
 */
const handleComplexSearch = async (command, respond) => {
  try {
    logger.debug(`${FILE_NAME}: Processing complex search command: ${JSON.stringify(command.text)}`);
    
    if (!command || !command.text) {
      logger.warn(`${FILE_NAME}: Empty command received for complex search`);
      await respond('Invalid command. Usage: /sigma-search where [conditions]');
      return;
    }
    
    // Extract query string
    let queryString = command.text.trim();
    let page = 1;
    let pageSize = MAX_RESULTS_PER_PAGE;
    
    // Check for pagination format: query page=X
    const pagingMatch = queryString.match(/(.+)\s+page=(\d+)$/i);
    if (pagingMatch) {
      queryString = pagingMatch[1].trim();
      page = parseInt(pagingMatch[2], 10) || 1;
      logger.debug(`${FILE_NAME}: Detected pagination request in complex search: page ${page}`);
    }
    
    // Check for page size format: query limit=X
    const limitMatch = queryString.match(/(.+)\s+limit=(\d+)$/i);
    if (limitMatch) {
      queryString = limitMatch[1].trim();
      pageSize = parseInt(limitMatch[2], 10) || MAX_RESULTS_PER_PAGE;
      // Ensure the page size is within reasonable limits
      pageSize = Math.min(Math.max(pageSize, 1), 100);
      logger.debug(`${FILE_NAME}: Detected page size request in complex search: limit ${pageSize}`);
    }
    
    logger.info(`${FILE_NAME}: Performing complex search with query: ${queryString}`);
    
    await respond({
      text: 'Processing complex search query... This may take a moment.',
      response_type: 'ephemeral'
    });
    
    // Perform the complex search
    const searchResult = await searchSigmaRulesComplex(queryString, page, pageSize);
    
    if (!searchResult.success) {
      logger.error(`${FILE_NAME}: Complex search failed: ${searchResult.message}`);
      await respond({
        text: `Search failed: ${searchResult.message}`,
        response_type: 'ephemeral'
      });
      return;
    }
    
    // Check if we have results
    if (!searchResult.results || searchResult.results.length === 0) {
      logger.warn(`${FILE_NAME}: No rules found matching complex query criteria`);
      await respond({
        text: `No rules found matching the specified criteria.`,
        response_type: 'ephemeral'
      });
      return;
    }
    
    // Generate blocks with pagination support
    let blocks;
    try {
      // Use the standard search result blocks but with a modified header
      blocks = getSearchResultBlocks(
        `Complex Query: ${queryString}`, 
        searchResult.results, 
        searchResult.pagination
      );
      
      // Replace the header to indicate it's a complex search
      // TODO: should be moved to dedicated block file
      if (blocks && blocks.length > 0) {
        blocks[0] = {
          type: "header",
          text: {
            type: "plain_text",
            text: `Sigma Rule Search Results - Query`,
            emoji: true
          }
        };
        
        // Add a description of the search criteria
        blocks.splice(1, 0, {
          type: "section",
          text: {
            type: "mrkdwn",
            text: `*Query:* \`${queryString}\``
          }
        });
      }
    } catch (blockError) {
      await handleError(blockError, `${FILE_NAME}: Complex search block generation`, respond, {
        responseType: 'ephemeral',
        customMessage: `Error generating results view: ${blockError.message}`
      });
      return;
    }
    
    // Respond with the search results
    await respond({
      blocks: blocks,
      response_type: 'ephemeral' // Complex searches are usually more specific to the user
    });
    
    logger.info(`${FILE_NAME}: Complex search response sent successfully with ${searchResult.results.length} results`);
  } catch (error) {
    await handleError(error, `${FILE_NAME}: Complex search handler`, respond, {
      responseType: 'ephemeral'
    });
  }
};

module.exports = {
  handleCommand,
  handleComplexSearch
};