/**
 * sigma_search_service.js
 * 
 * This service provides functionality for searching Sigma rules by keywords and complex queries.
 * It processes search results and returns them in a structured format.
 * Supports pagination for large result sets.
 */

const { searchRules, searchRulesComplex } = require('../../sigma_db/queries');
const { parseComplexQuery } = require('../../lang/query_parser');
const logger = require('../../utils/logger');
const { convertSigmaRule } = require('./sigma_converter_service');
const { getFileName } = require('../../utils/file_utils');
const FILE_NAME = getFileName(__filename);

/**
 * Searches for Sigma rules by keyword and processes the results
 * Returns a structured result object with success status and paginated results
 * 
 * @param {string} keyword - The keyword to search for
 * @param {number} page - Page number (1-based index, default: 1)
 * @param {number} pageSize - Number of results per page (default: 10)
 * @returns {Promise<Object>} Result object with success flag and processed results with pagination info
 */
async function searchSigmaRules(keyword, page = 1, pageSize = 10) {
  if (!keyword || typeof keyword !== 'string') {
    logger.warn(`${FILE_NAME}: Cannot search rules: Missing or invalid keyword`);
    return {
      success: false,
      message: 'Missing or invalid search keyword'
    };
  }
  
  // Validate pagination parameters
  if (typeof page !== 'number' || page < 1) {
    logger.warn(`${FILE_NAME}: Invalid page number: ${page}, defaulting to 1`);
    page = 1;
  }
  
  if (typeof pageSize !== 'number' || pageSize < 1 || pageSize > 100) {
    logger.warn(`${FILE_NAME}: Invalid page size: ${pageSize}, defaulting to 10`);
    pageSize = 10;
  }
  
  // Trim the keyword to prevent accidental whitespace issues
  const trimmedKeyword = keyword.trim();
  if (trimmedKeyword.length === 0) {
    logger.warn(`${FILE_NAME}: Cannot search rules: Empty keyword after trimming`);
    return {
      success: false,
      message: 'Search keyword cannot be empty'
    };
  }
  
  // Calculate the offset based on page number
  const offset = (page - 1) * pageSize;
  
  logger.info(`${FILE_NAME}: Searching for Sigma rules with keyword: "${trimmedKeyword}" (page ${page}, size ${pageSize}, offset ${offset})`);
  
  try {
    // Pass pageSize and offset to the database query
    const searchResult = await searchRules(trimmedKeyword, pageSize, offset);
    
    // Defensive handling of possible return formats
    let allResults = [];
    let totalCount = 0;
    
    // Log what we actually received for debugging
    logger.debug(`${FILE_NAME}: Search result type: ${typeof searchResult}, isArray: ${Array.isArray(searchResult)}`);
    
    // Handle different possible return formats
    if (searchResult) {
      if (Array.isArray(searchResult)) {
        // Direct array of results
        allResults = searchResult;
        logger.debug(`${FILE_NAME}: Received array of ${allResults.length} results`);
      } else if (typeof searchResult === 'object') {
        // Object with results property
        if (Array.isArray(searchResult.results)) {
          allResults = searchResult.results;
          totalCount = searchResult.totalCount || 0;
          logger.debug(`${FILE_NAME}: Received object with ${allResults.length} results of ${totalCount} total matches`);
        } else if (searchResult.totalCount !== undefined) {
          // Object might have a different structure
          totalCount = searchResult.totalCount;
          logger.debug(`${FILE_NAME}: Received object with totalCount ${totalCount}`);
        }
      }
    }
    
    // Log what we extracted
    logger.debug(`${FILE_NAME}: Extracted ${allResults.length} results for page ${page} of total ${totalCount}`);
    
    if (allResults.length === 0 && totalCount === 0) {
      logger.info(`${FILE_NAME}: No rules found matching "${trimmedKeyword}"`);
      return {
        success: true,
        results: [],
        message: `No rules found matching "${trimmedKeyword}"`,
        pagination: {
          currentPage: 1,
          pageSize: pageSize,
          totalPages: 0,
          totalResults: 0,
          hasMore: false
        }
      };
    }
    
    // Calculate total pages and pagination info based on total count from database
    const totalPages = Math.ceil(totalCount / pageSize);
    const hasMore = (offset + pageSize) < totalCount;
    
    // Check if the requested page is valid
    if (offset >= totalCount && totalCount > 0) {
      // Return empty results but with pagination info
      logger.warn(`${FILE_NAME}: Page ${page} exceeds available results (total: ${totalCount})`);
      return {
        success: true,
        results: [],
        message: `No results on page ${page}. Try a previous page.`,
        pagination: {
          currentPage: page,
          pageSize: pageSize,
          totalPages: totalPages,
          totalResults: totalCount,
          hasMore: false
        }
      };
    }
    
    // If we have results, include them with pagination info
    logger.debug(`${FILE_NAME}: Returning ${allResults.length} results with pagination info (page ${page}/${totalPages}, total: ${totalCount})`);
    
    return {
      success: true,
      results: allResults,
      count: allResults.length,
      pagination: {
        currentPage: page,
        pageSize: pageSize,
        totalPages: totalPages,
        totalResults: totalCount,
        hasMore: hasMore
      }
    };
  } catch (error) {
    logger.error(`${FILE_NAME}: Error searching for rules: ${error.message}`);
    logger.debug(`${FILE_NAME}: Error stack: ${error.stack}`);
    return {
      success: false,
      message: `Error searching for rules: ${error.message}`
    };
  }
}

/**
 * Searches for Sigma rules using complex query conditions
 * Supports filtering by title, logsource, tags, dates, and more
 * 
 * @param {string} queryString - The complex query string to parse
 * @param {number} page - Page number (1-based index, default: 1)
 * @param {number} pageSize - Number of results per page (default: 10)
 * @returns {Promise<Object>} Result object with success flag and processed results
 */
async function searchSigmaRulesComplex(queryString, page = 1, pageSize = 10) {
  if (!queryString || typeof queryString !== 'string') {
    logger.warn(`${FILE_NAME}: Cannot perform complex search: Missing or invalid query string`);
    return {
      success: false,
      message: 'Missing or invalid complex query'
    };
  }
  
  // Validate pagination parameters
  if (typeof page !== 'number' || page < 1) {
    logger.warn(`${FILE_NAME}: Invalid page number: ${page}, defaulting to 1`);
    page = 1;
  }
  
  if (typeof pageSize !== 'number' || pageSize < 1 || pageSize > 100) {
    logger.warn(`${FILE_NAME}: Invalid page size: ${pageSize}, defaulting to 10`);
    pageSize = 10;
  }
  
  // Calculate the offset based on page number
  const offset = (page - 1) * pageSize;
  
  logger.info(`${FILE_NAME}: Performing complex search with query: "${queryString}" (page ${page}, size ${pageSize})`);
  
  try {
    // Parse the complex query string
    const parsedQuery = parseComplexQuery(queryString);
    
    if (!parsedQuery.valid) {
      logger.warn(`${FILE_NAME}: Invalid complex query: ${parsedQuery.error}`);
      return {
        success: false,
        message: `Invalid query: ${parsedQuery.error}`
      };
    }
    
    // Perform the database search with the parsed query
    const searchResult = await searchRulesComplex(parsedQuery, pageSize, offset);
    
    // Defensive handling of possible return formats
    let allResults = [];
    let totalCount = 0;
    
    // Handle search results
    if (searchResult) {
      if (Array.isArray(searchResult.results)) {
        allResults = searchResult.results;
        totalCount = searchResult.totalCount || 0;
      }
    }
    
    if (allResults.length === 0) {
      return {
        success: true,
        results: [],
        message: `No rules found matching the complex query criteria`,
        pagination: {
          currentPage: page,
          pageSize: pageSize,
          totalPages: Math.ceil(totalCount / pageSize),
          totalResults: totalCount,
          hasMore: false
        }
      };
    }
    
    // Calculate pagination info
    const totalPages = Math.ceil(totalCount / pageSize);
    const hasMore = (offset + pageSize) < totalCount;
    
    return {
      success: true,
      results: allResults,
      count: allResults.length,
      query: parsedQuery,
      pagination: {
        currentPage: page,
        pageSize: pageSize,
        totalPages: totalPages,
        totalResults: totalCount,
        hasMore: hasMore
      }
    };
  } catch (error) {
    logger.error(`${FILE_NAME}: Error in complex search: ${error.message}`);
    logger.debug(`${FILE_NAME}: Error stack: ${error.stack}`);
    return {
      success: false,
      message: `Error performing complex search: ${error.message}`
    };
  }
}

/**
 * Enhanced search that returns fully converted rule objects with pagination support
 * This is a more expensive operation than basic search
 * 
 * @param {string} keyword - The keyword to search for
 * @param {number} page - Page number (1-based index, default: 1)
 * @param {number} pageSize - Number of results per page (default: 10) 
 * @returns {Promise<Object>} Result object with success flag and fully converted rule objects with pagination info
 */
async function searchAndConvertRules(keyword, page = 1, pageSize = 10) {
  try {
    // First perform a basic search with pagination
    const searchResult = await searchSigmaRules(keyword, page, pageSize);
    
    if (!searchResult.success || !searchResult.results || searchResult.results.length === 0) {
      return searchResult;
    }
    
    logger.debug(`${FILE_NAME}: Converting ${searchResult.results.length} search results to full rule objects`);
    
    // Convert each result to a full rule object
    const convertedResults = [];
    for (const result of searchResult.results) {
      try {
        const conversionResult = await convertSigmaRule(result.id);
        if (conversionResult.success && conversionResult.rule) {
          convertedResults.push(conversionResult.rule);
        } else {
          logger.warn(`${FILE_NAME}: Failed to convert rule ${result.id}: ${conversionResult.message || 'Unknown error'}`);
        }
      } catch (conversionError) {
        logger.error(`${FILE_NAME}: Error converting rule ${result.id}: ${conversionError.message}`);
      }
    }
    
    logger.info(`${FILE_NAME}: Successfully converted ${convertedResults.length} of ${searchResult.results.length} search results`);
    
    // Include the pagination information from the search results
    return {
      success: true,
      results: convertedResults,
      count: convertedResults.length,
      originalCount: searchResult.results.length,
      pagination: searchResult.pagination
    };
  } catch (error) {
    logger.error(`${FILE_NAME}: Error in searchAndConvertRules: ${error.message}`);
    logger.debug(`${FILE_NAME}: Error stack: ${error.stack}`);
    return {
      success: false,
      message: `Error searching and converting rules: ${error.message}`
    };
  }
}

module.exports = {
  searchSigmaRules,
  searchSigmaRulesComplex,
  searchAndConvertRules
};