/**
 * stats-debug.js
 * Functions for database statistics and debugging
 */

const { getDbConnection } = require('../sigma_db_connection');
const logger = require('../../utils/logger');
const { DB_PATH } = require('../../config/appConfig');
const path = require('path');
const { getFileName } = require('../../utils/file_utils');
const FILE_NAME = getFileName(__filename);

/**
 * Debug function to retrieve detailed information about a rule's content
 * Useful for diagnosing issues with rule retrieval and content parsing
 * 
 * @param {string} ruleId - The ID of the rule to debug
 * @returns {Promise<Object|null>} Object containing debug information or null on error
 */
async function debugRuleContent(ruleId) {
  if (!ruleId) {
    logger.warn(`${FILE_NAME}: Cannot debug rule: Missing rule ID`);
    return null;
  }

  let db;
  try {
    db = await getDbConnection();

    const absolutePath = path.resolve(DB_PATH);
    logger.debug(`${FILE_NAME}: Debug function connecting to DB at path: ${absolutePath}`);

    // Get raw rule record
    const rule = await db.get('SELECT id, file_path, length(content) as content_length, typeof(content) as content_type FROM sigma_rules WHERE id = ?', [ruleId]);

    if (!rule) {
      logger.warn(`${FILE_NAME}: Rule with ID ${ruleId} not found during debug operation`);
      return { error: 'Rule not found', ruleId };
    }

    // Return just the rule information without the undefined variables
    return {
      rule,
      ruleId
    };
  } catch (error) {
    logger.error(`${FILE_NAME}: Debug error for rule ${ruleId}: ${error.message}`);
    logger.debug(`${FILE_NAME}: Debug error stack: ${error.stack}`);
    return {
      error: error.message,
      stack: error.stack,
      ruleId
    };
  } finally {
    if (db) {
      try {
        await db.close();
        logger.debug(`${FILE_NAME}: Database connection closed after debug operation`);
      } catch (closeError) {
        logger.warn(`${FILE_NAME}: Error closing database after debug: ${closeError.message}`);
      }
    }
  }
}

/**
 * Get the raw YAML content of a Sigma rule
 * Retrieves the content field from the database which should contain YAML
 * 
 * @param {string} ruleId - The ID of the rule
 * @returns {Promise<Object>} Object with success flag and content or error message
 */
async function getRuleYamlContent(ruleId) {
  if (!ruleId) {
    logger.warn(`${FILE_NAME}: Cannot get YAML content: Missing rule ID`);
    return { success: false, message: 'Missing rule ID' };
  }

  let db;
  try {
    logger.info(`${FILE_NAME}: Fetching YAML content for rule: ${ruleId}`);
    logger.debug(`${FILE_NAME}: Rule ID type: ${typeof ruleId}, length: ${ruleId.length}`);

    db = await getDbConnection();
    logger.debug(`${FILE_NAME}: Connected to database for YAML retrieval`);

    // Debug query before execution
    const debugResult = await db.get('SELECT id, typeof(content) as content_type, length(content) as content_length FROM sigma_rules WHERE id = ?', [ruleId]);
    logger.debug(`${FILE_NAME}: Debug query result: ${JSON.stringify(debugResult || 'not found')}`);

    if (!debugResult) {
      logger.warn(`${FILE_NAME}: Rule with ID ${ruleId} not found in debug query`);
      return { success: false, message: 'Rule not found' };
    }

    // Get actual content
    const rule = await new Promise((resolve, reject) => {
      db.get('SELECT content FROM sigma_rules WHERE id = ?', [ruleId], (err, row) => {
        if (err) {
          logger.error(`${FILE_NAME}: Content query error: ${err.message}`);
          reject(err);
        } else {
          resolve(row || null);
        }
      });
    });
    logger.debug(`${FILE_NAME}: Content query result for ${ruleId}: ${rule ? 'Found' : 'Not found'}`);

    if (!rule) {
      logger.warn(`${FILE_NAME}: Rule with ID ${ruleId} not found in content query`);
      return { success: false, message: 'Rule not found' };
    }

    if (!rule.content) {
      logger.warn(`${FILE_NAME}: Rule with ID ${ruleId} found but has no content`);
      return { success: false, message: 'Rule found but content is empty' };
    }

    logger.debug(`${FILE_NAME}: Content retrieved successfully for ${ruleId}, type: ${typeof rule.content}, length: ${rule.content.length}`);

    return { success: true, content: rule.content };
  } catch (error) {
    logger.error(`${FILE_NAME}: Error retrieving YAML content for ${ruleId}: ${error.message}`);
    logger.debug(`${FILE_NAME}: YAML retrieval error stack: ${error.stack}`);
    return { success: false, message: `Error retrieving YAML: ${error.message}` };
  } finally {
    if (db) {
      try {
        await db.close();
        logger.debug(`${FILE_NAME}: Database connection closed after YAML retrieval`);
      } catch (closeError) {
        logger.warn(`${FILE_NAME}: Error closing database after YAML retrieval: ${closeError.message}`);
      }
    }
  }
}

/**
 * Get statistics about Sigma rules in the database
 * Collects counts, categories, and other aggregate information
 * 
 * @returns {Promise<Object>} Object with various statistics about the rules
 */
async function getStatsFromDatabase() {
  let db;
  try {
    db = await getDbConnection();
    logger.debug(`${FILE_NAME}: Connected to database for statistics`);

    // Get total rule count
    const totalRulesResult = await new Promise((resolve, reject) => {
      db.get('SELECT COUNT(*) as count FROM sigma_rules', (err, row) => {
        if (err) reject(err);
        else resolve(row);
      });
    });
    const totalRules = totalRulesResult.count;

    // Get last update time
    const lastUpdateResult = await new Promise((resolve, reject) => {
      db.get('SELECT MAX(date) as last_update FROM sigma_rules', (err, row) => {
        if (err) reject(err);
        else resolve(row);
      });
    });
    const lastUpdate = lastUpdateResult.last_update;

    // Get rules by log source count (Windows, Linux, macOS)
    const windowsRulesResult = await new Promise((resolve, reject) => {
      db.get(`
        SELECT COUNT(DISTINCT rule_id) as count 
        FROM rule_parameters 
        WHERE param_name = 'logsource' AND 
        param_value LIKE '%"product":"windows"%'`,
        (err, row) => {
          if (err) reject(err);
          else resolve(row);
        });
    });
    const windowsRules = windowsRulesResult.count || 0;

    const linuxRulesResult = await new Promise((resolve, reject) => {
      db.get(`
        SELECT COUNT(DISTINCT rule_id) as count 
        FROM rule_parameters 
        WHERE param_name = 'logsource' AND 
        param_value LIKE '%"product":"linux"%'`,
        (err, row) => {
          if (err) reject(err);
          else resolve(row);
        });
    });
    const linuxRules = linuxRulesResult.count || 0;

    const macosRulesResult = await new Promise((resolve, reject) => {
      db.get(`
        SELECT COUNT(DISTINCT rule_id) as count 
        FROM rule_parameters 
        WHERE param_name = 'logsource' AND 
        param_value LIKE '%"product":"macos"%'`,
        (err, row) => {
          if (err) reject(err);
          else resolve(row);
        });
    });
    const macosRules = macosRulesResult.count || 0;

    // Get rules by severity level
    const severityStats = await new Promise((resolve, reject) => {
      db.all(`
        SELECT param_value AS level, COUNT(DISTINCT rule_id) as count 
        FROM rule_parameters 
        WHERE param_name = 'level' 
        GROUP BY param_value 
        ORDER BY 
          CASE 
            WHEN param_value = 'critical' THEN 1 
            WHEN param_value = 'high' THEN 2 
            WHEN param_value = 'medium' THEN 3 
            WHEN param_value = 'low' THEN 4 
            WHEN param_value = 'informational' THEN 5 
            ELSE 6 
          END`,
        (err, rows) => {
          if (err) reject(err);
          else resolve(rows);
        });
    });

    // Get top 5 rule authors
    const topAuthors = await new Promise((resolve, reject) => {
      db.all(`
        SELECT param_value AS author, COUNT(*) as count 
        FROM rule_parameters 
        WHERE param_name = 'author' 
        GROUP BY param_value 
        ORDER BY count DESC 
        LIMIT 5`,
        (err, rows) => {
          if (err) reject(err);
          else resolve(rows);
        });
    });

    // Get empty content count (rules with missing YAML)
    const emptyContentResult = await new Promise((resolve, reject) => {
      db.get(`
        SELECT COUNT(*) as count 
        FROM sigma_rules 
        WHERE content IS NULL OR content = ''`,
        (err, row) => {
          if (err) reject(err);
          else resolve(row);
        });
    });
    const emptyContentCount = emptyContentResult.count;

    // Get MITRE ATT&CK tactics statistics
    const mitreStats = await new Promise((resolve, reject) => {
      db.all(`
        SELECT param_value AS tag, COUNT(DISTINCT rule_id) as count 
        FROM rule_parameters 
        WHERE param_name = 'tags' AND param_value LIKE 'attack.%' 
        GROUP BY param_value 
        ORDER BY count DESC 
        LIMIT 10`,
        (err, rows) => {
          if (err) reject(err);
          else resolve(rows);
        });
    });

    // Format MITRE tactics for display
    const formattedMitreTactics = mitreStats.map(item => {
      const tactic = item.tag.substring(7); // Remove 'attack.' prefix
      return {
        tactic: tactic,
        count: item.count
      };
    });

    // Compile all statistics
    const stats = {
      totalRules,
      lastUpdate,
      operatingSystems: {
        windows: windowsRules,
        linux: linuxRules,
        macos: macosRules,
        other: totalRules - (windowsRules + linuxRules + macosRules)
      },
      severityLevels: severityStats.map(s => ({ level: s.level, count: s.count })),
      topAuthors: topAuthors.map(a => ({ name: a.author, count: a.count })),
      databaseHealth: {
        emptyContentCount,
        contentPercentage: totalRules > 0 ? Math.round(((totalRules - emptyContentCount) / totalRules) * 100) : 0
      },
      mitreTactics: formattedMitreTactics
    };

    return {
      success: true,
      stats
    };
  } catch (error) {
    logger.error(`${FILE_NAME}: Error retrieving statistics: ${error.message}`);
    logger.debug(`${FILE_NAME}: Error stack: ${error.stack}`);
    return {
      success: false,
      message: `Error retrieving statistics: ${error.message}`
    };
  } finally {
    if (db) {
      try {
        await new Promise((resolve) => db.close(() => resolve()));
        logger.debug(`${FILE_NAME}: Database connection closed after statistics retrieval`);
      } catch (closeError) {
        logger.warn(`${FILE_NAME}: Error closing database: ${closeError.message}`);
      }
    }
  }
}

module.exports = {
  debugRuleContent,
  getRuleYamlContent,
  getStatsFromDatabase
};