//
// sigma_db_initialize.js
// This script initializes the Sigma database by importing rules from the Sigma repository.
// and creating the SQLite database.
//
const fs = require('fs');
const util = require('util');
const sqlite3 = require('sqlite3').verbose();
const yaml = require('js-yaml');
const glob = util.promisify(require('glob'));

const logger = require('../utils/logger');
const { SIGMA_REPO_DIR, DB_PATH } = require('../config/appConfig');
const { updateSigmaRepo } = require('../services/sigma/sigma_repository_service');

const { getFileName } = require('../utils/file_utils');
const FILE_NAME = getFileName(__filename);


// Create database connection
function createDbConnection() {
  return new Promise((resolve, reject) => {
    const db = new sqlite3.Database(DB_PATH, (err) => {
      if (err) {
        reject(err);
        return;
      }
      
      logger.info(`${FILE_NAME}: Connected to SQLite database at ${DB_PATH}`);
      
      // CRITICAL FIX: Enable foreign key constraints
      db.run('PRAGMA foreign_keys = ON;', (pragmaErr) => {
        if (pragmaErr) {
          logger.error(`${FILE_NAME}: Failed to enable foreign key constraints: ${pragmaErr.message}`);
          reject(pragmaErr);
        } else {
          logger.info(`${FILE_NAME}: Foreign key constraints enabled`);
          resolve(db);
        }
      });
    });
  });
}

// Initialize database schema
async function initializeDatabase(db) {
  return new Promise((resolve, reject) => {
    // Drop existing tables if they exist
    db.run('DROP TABLE IF EXISTS rule_parameters', (err) => {
      if (err) {
        reject(err);
        return;
      }
      
      db.run('DROP TABLE IF EXISTS sigma_rules', (err) => {
        if (err) {
          reject(err);
          return;
        }
        
        // Drop FTS table if exists
        db.run('DROP TABLE IF EXISTS rule_search', (err) => {
          if (err) {
            reject(err);
            return;
          }
          
          // Create rules table with basic information
          const createRulesTableSql = `
            CREATE TABLE sigma_rules (
              id TEXT PRIMARY KEY,
              file_path TEXT,
              content TEXT,
              date DATETIME DEFAULT CURRENT_TIMESTAMP
            )
          `;
          
          db.run(createRulesTableSql, (err) => {
            if (err) {
              reject(err);
              return;
            }
            
            // Create rule_parameters table for individual parameters
            const createParamsTableSql = `
              CREATE TABLE rule_parameters (
                id INTEGER PRIMARY KEY AUTOINCREMENT,
                rule_id TEXT,
                param_name TEXT,
                param_value TEXT,
                param_type TEXT,
                FOREIGN KEY (rule_id) REFERENCES sigma_rules(id) ON DELETE CASCADE
              )
            `;
            
            db.run(createParamsTableSql, (err) => {
              if (err) {
                reject(err);
              } else {
                logger.info(`${FILE_NAME}: Database schema initialized`);
                resolve();
              }
            });
          });
        });
      });
    });
  });
}

// Create FTS5 virtual table for full-text search
async function createFtsTable(db) {
  return new Promise((resolve, reject) => {
    logger.info(`${FILE_NAME}: Creating FTS5 virtual table for full-text search`);
    
    // Create the FTS5 virtual table
    const createFtsTableSql = `
      CREATE VIRTUAL TABLE IF NOT EXISTS rule_search USING fts5(
        rule_id, 
        title, 
        description, 
        logsource, 
        tags, 
        author, 
        level, 
        content, 
        tokenize="unicode61"
      );
    `;
    
    db.run(createFtsTableSql, (err) => {
      if (err) {
        logger.error(`${FILE_NAME}: Failed to create FTS5 table: ${err.message}`);
        reject(err);
      } else {
        logger.info(`${FILE_NAME}: FTS5 virtual table created successfully`);
        resolve();
      }
    });
  });
}

// Populate FTS table with rule data for full-text search
async function populateFtsTable(db) {
  return new Promise((resolve, reject) => {
    logger.info(`${FILE_NAME}: Populating FTS5 table with rule data`);
    
    // Insert query that aggregates data from both tables
    const populateFtsSql = `
      INSERT INTO rule_search(rule_id, title, description, logsource, tags, author, level, content)
      SELECT 
        r.id,
        (SELECT param_value FROM rule_parameters WHERE rule_id = r.id AND param_name = 'title' LIMIT 1),
        (SELECT param_value FROM rule_parameters WHERE rule_id = r.id AND param_name = 'description' LIMIT 1),
        (SELECT param_value FROM rule_parameters WHERE rule_id = r.id AND param_name = 'logsource' LIMIT 1),
        (SELECT param_value FROM rule_parameters WHERE rule_id = r.id AND param_name = 'tags' LIMIT 1),
        (SELECT param_value FROM rule_parameters WHERE rule_id = r.id AND param_name = 'author' LIMIT 1),
        (SELECT param_value FROM rule_parameters WHERE rule_id = r.id AND param_name = 'level' LIMIT 1),
        r.content
      FROM sigma_rules r
    `;
    
    db.run(populateFtsSql, (err) => {
      if (err) {
        logger.error(`${FILE_NAME}: Failed to populate FTS5 table: ${err.message}`);
        reject(err);
      } else {
        logger.info(`${FILE_NAME}: FTS5 table populated successfully`);
        resolve();
      }
    });
  });
}

// Determine if a YAML document is a Sigma rule
function isSigmaRule(doc) {
  // Check for essential Sigma rule properties
  return doc && doc.id && (
    doc.detection || 
    doc.logsource || 
    doc.title
  );
}

// Parse a Sigma YAML file and extract relevant fields
function parseRuleFile(filePath) {
  try {
    // Read the file content
    const content = fs.readFileSync(filePath, 'utf8');
        
    // Try to load as a multi-document YAML
    let documents = [];
    try {
      yaml.loadAll(content, (doc) => {
        if (doc) documents.push(doc);
      });
    } catch (e) {
      // If multi-document parsing fails, try as a single document
      logger.warn(`${FILE_NAME}: Multi-document parsing failed for ${filePath}: ${e.message}`);
      try {
        const doc = yaml.load(content);
        if (doc) documents.push(doc);
      } catch (singleError) {
        logger.error(`Failed to parse ${filePath} as YAML: ${singleError.message}`);
      }
    }
        
    // Filter to only include documents that look like Sigma rules
    const ruleDocuments = documents.filter(isSigmaRule);
        
    // Return array of rule objects
    return ruleDocuments.map(rule => {
      // Ensure rule.id exists - this is critical
      if (!rule.id) {
        logger.warn(`${FILE_NAME}: Rule in ${filePath} has no ID, skipping`);
        return null;
      }
      
      return {
        id: rule.id,
        file_path: filePath,
        content: content,  // Explicitly assign content
        parameters: extractParameters(rule)
      };
    }).filter(rule => rule !== null); // Remove null rules
  } catch (error) {
    logger.error(`${FILE_NAME}: Error parsing ${filePath}: ${error.message}`);
    return [];
  }
}

// Extract all parameters from a rule object, including nested ones
function extractParameters(rule) {
  const params = [];
  
  function processValue(name, value, parentKey = '') {
    const fullKey = parentKey ? `${parentKey}.${name}` : name;
    
    if (value === null || value === undefined) {
      params.push({
        param_name: fullKey,
        param_value: '',
        param_type: 'null'
      });
    } else if (typeof value === 'object' && !Array.isArray(value)) {
      // For objects, add a parameter for the object itself
      params.push({
        param_name: fullKey,
        param_value: JSON.stringify(value),
        param_type: 'object'
      });
      
      // Then process all its properties
      Object.entries(value).forEach(([k, v]) => {
        processValue(k, v, fullKey);
      });
    } else if (Array.isArray(value)) {
      // For arrays, add a parameter for the array itself
      params.push({
        param_name: fullKey,
        param_value: JSON.stringify(value),
        param_type: 'array'
      });
      
      // And also add individual array elements
      value.forEach((item, index) => {
        if (typeof item === 'object' && item !== null) {
          processValue(index.toString(), item, fullKey);
        } else {
          params.push({
            param_name: `${fullKey}[${index}]`,
            param_value: String(item),
            param_type: typeof item
          });
        }
      });
    } else {
      // Handle primitive types
      params.push({
        param_name: fullKey,
        param_value: String(value),
        param_type: typeof value
      });
    }
  }
  
  // Process all properties in the rule
  Object.entries(rule).forEach(([key, value]) => {
    processValue(key, value);
  });
  
  return params;
}

// Import rules into database
async function importRules(db) {
  // Find all YAML files in the entire repository
  logger.info('${FILE_NAME}: Looking for rule files...');
  
  // Get all YAML files - include both .yml and .yaml extensions
  const files = await glob(`${SIGMA_REPO_DIR}/**/*.{yml,yaml}`);
  logger.info(`${FILE_NAME}: Found ${files.length} total YAML files`);
  
  // Prepare insert statements with explicit parameter naming
  const insertRuleStmt = db.prepare(`
    INSERT OR REPLACE INTO sigma_rules (id, file_path, content)
    VALUES (?, ?, ?)
  `);
  
  const insertParamStmt = db.prepare(`
    INSERT INTO rule_parameters (rule_id, param_name, param_value, param_type)
    VALUES (?, ?, ?, ?)
  `);
  
  // Process each file
  let importedRuleCount = 0;
  let importedParamCount = 0;
  let skippedFileCount = 0;
  let errorCount = 0;
  
  // Keep track of content status
  let rulesWithContent = 0;
  let rulesWithoutContent = 0;
  
  for (const file of files) {
    try {
      const rules = parseRuleFile(file);
      
      if (rules.length === 0) {
        skippedFileCount++;
        continue;
      }
      
      for (const rule of rules) {
        // Begin transaction
        await new Promise((resolve, reject) => {
          db.run('BEGIN TRANSACTION', (err) => {
            if (err) reject(err);
            else resolve();
          });
        });
        
        let transactionSuccessful = true;
        
        try {
          // Debug: Check content before insertion
          const hasContent = !!(rule.content && rule.content.length > 0);
          
          if (hasContent) {
            rulesWithContent++;
          } else {
            rulesWithoutContent++;
            logger.warn(`Rule ${rule.id} has no content!`);
            // CRITICAL FIX: Ensure empty content is handled as empty string, not null
            rule.content = '';
          }
          
          // Insert rule with explicit parameters
          await new Promise((resolve, reject) => {
            insertRuleStmt.run(
              rule.id,                // Parameter 1: id
              rule.file_path,         // Parameter 2: file_path
              rule.content || '',     // Parameter 3: content (ensure not null)
              (err) => {
                if (err) {
                  logger.error(`Error inserting rule ${rule.id}: ${err.message}`);
                  transactionSuccessful = false;
                  reject(err);
                } else {
                  resolve();
                }
              }
            );
          });
          
          // CRITICAL FIX: Only insert parameters if rule insertion was successful
          if (transactionSuccessful) {
            // Insert parameters
            for (const param of rule.parameters) {
              try {
                await new Promise((resolve, reject) => {
                  insertParamStmt.run(
                    rule.id,              // Parameter 1: rule_id
                    param.param_name,     // Parameter 2: param_name
                    param.param_value,    // Parameter 3: param_value
                    param.param_type,     // Parameter 4: param_type
                    (err) => {
                      if (err) {
                        logger.error(`${FILE_NAME}: Error inserting parameter ${param.param_name} for rule ${rule.id}: ${err.message}`);
                        transactionSuccessful = false;
                        reject(err);
                      } else {
                        resolve();
                      }
                    }
                  );
                });
                
                // CRITICAL FIX: Stop processing parameters if any insertion fails
                if (!transactionSuccessful) {
                  break;
                }
                
                importedParamCount++;
              } catch (paramError) {
                logger.error(`${FILE_NAME}: Error processing parameter: ${paramError.message}`);
                transactionSuccessful = false;
                break;
              }
            }
          }
          
          // CRITICAL FIX: Only commit if everything was successful
          if (transactionSuccessful) {
            // Commit transaction
            await new Promise((resolve, reject) => {
              db.run('COMMIT', (err) => {
                if (err) {
                  transactionSuccessful = false;
                  reject(err);
                } else {
                  resolve();
                }
              });
            });
            
            importedRuleCount++;
            
            // Log progress every 100 rules
            if (importedRuleCount % 100 === 0) {
              logger.info(`${FILE_NAME}: Imported ${importedRuleCount} rules with ${importedParamCount} parameters, ${rulesWithContent} have content, ${rulesWithoutContent} missing content`);
            }
          } else {
            // Rollback transaction if not successful
            await new Promise((resolve) => {
              db.run('ROLLBACK', () => resolve());
            });
            
            errorCount++;
          }
        } catch (error) {
          // Rollback transaction on error
          await new Promise((resolve) => {
            db.run('ROLLBACK', () => resolve());
          });
          
          logger.error(`${FILE_NAME}: Error importing rule ${rule.id} from ${file}: ${error.message}`);
          errorCount++;
        }
      }
    } catch (error) {
      logger.error(`${FILE_NAME}: Error processing file ${file}: ${error.message}`);
      errorCount++;
    }
  }
  
  insertRuleStmt.finalize();
  insertParamStmt.finalize();
  
  logger.info(`${FILE_NAME}: Import summary: ${importedRuleCount} rules imported with ${importedParamCount} parameters, ${skippedFileCount} files skipped, ${errorCount} errors`);
  logger.info(`${FILE_NAME}: Content status: ${rulesWithContent} rules with content, ${rulesWithoutContent} rules without content`);
  
  // Run diagnostics after import
  await diagnoseContentImport(db);
}

// Diagnose rule content import issues
async function diagnoseContentImport(db) {
  logger.info(`${FILE_NAME}: Starting content import diagnosis...`);
  
  // Check total rule count
  const ruleCount = await new Promise((resolve, reject) => {
    db.get('SELECT COUNT(*) as count FROM sigma_rules', (err, row) => {
      if (err) reject(err);
      else resolve(row ? row.count : 0);
    });
  });
  
  logger.info(`${FILE_NAME}: Total rules in database: ${ruleCount}`);
  
  // Sample a few rules
  const sampleRules = await new Promise((resolve, reject) => {
    db.all('SELECT id, file_path, length(content) as content_length FROM sigma_rules LIMIT 5', (err, rows) => {
      if (err) reject(err);
      else resolve(rows || []);
    });
  });
  
  logger.info(`${FILE_NAME}: Sample rule content lengths: ${JSON.stringify(sampleRules.map(r => ({ id: r.id, content_length: r.content_length })))}`);
  
  // Check the content column type
  const tableInfo = await new Promise((resolve, reject) => {
    db.all('PRAGMA table_info(sigma_rules)', (err, rows) => {
      if (err) reject(err);
      else resolve(rows || []);
    });
  });
  
  const contentColumn = tableInfo.find(col => col.name === 'content');
  logger.info(`${FILE_NAME}: Content column info: ${JSON.stringify(contentColumn)}`);
  
  // Check a rule's content explicitly
  if (sampleRules.length > 0) {
    const firstRuleId = sampleRules[0].id;
    const ruleContent = await new Promise((resolve, reject) => {
      db.get('SELECT content FROM sigma_rules WHERE id = ?', [firstRuleId], (err, row) => {
        if (err) reject(err);
        else resolve(row ? (row.content || null) : null);
      });
    });
    
    logger.info(`${FILE_NAME}: First rule content check: id=${firstRuleId}, has_content=${ruleContent !== null}, type=${typeof ruleContent}, length=${ruleContent ? ruleContent.length : 0}`);
    
    if (ruleContent) {
      logger.info(`Content sample: ${ruleContent.substring(0, 100)}...`);
    }
  }
  
  // Look for missing parameters
  const paramCount = await new Promise((resolve, reject) => {
    db.get('SELECT COUNT(*) as count FROM rule_parameters', (err, row) => {
      if (err) reject(err);
      else resolve(row ? row.count : 0);
    });
  });
  
  logger.info(`${FILE_NAME}: Total parameters: ${paramCount}`);
  
  // CRITICAL FIX: Check for orphaned parameters
  const orphanedParamCount = await new Promise((resolve, reject) => {
    db.get(`
      SELECT COUNT(*) as count 
      FROM rule_parameters p 
      LEFT JOIN sigma_rules r ON p.rule_id = r.id 
      WHERE r.id IS NULL
    `, (err, row) => {
      if (err) reject(err);
      else resolve(row ? row.count : 0);
    });
  });
  
  logger.info(`${FILE_NAME}: Orphaned parameters (should be 0): ${orphanedParamCount}`);
  
  if (orphanedParamCount > 0) {
    logger.error(`Found ${orphanedParamCount} orphaned parameters! This indicates a foreign key constraint violation.`);
  }
  
  return {
    rule_count: ruleCount,
    sample_rules: sampleRules,
    content_column: contentColumn,
    param_count: paramCount,
    orphaned_param_count: orphanedParamCount
  };
}

// Create indexes on the database for better performance
async function createIndexes(db) {
  return new Promise((resolve, reject) => {
    const indexes = [
      'CREATE INDEX IF NOT EXISTS idx_param_rule_id ON rule_parameters(rule_id)',
      'CREATE INDEX IF NOT EXISTS idx_param_name ON rule_parameters(param_name)',
      'CREATE INDEX IF NOT EXISTS idx_param_value ON rule_parameters(param_value)',
      'CREATE INDEX IF NOT EXISTS idx_param_type ON rule_parameters(param_type)'
    ];
    
    let completed = 0;
    
    for (const indexSql of indexes) {
      db.run(indexSql, (err) => {
        if (err) {
          reject(err);
        } else {
          completed++;
          if (completed === indexes.length) {
            logger.info('Database indexes created');
            resolve();
          }
        }
      });
    }
  });
}

// Main function
async function main() {
  try {
    // Update Sigma repository
    const repoUpdated = await updateSigmaRepo();
    if (!repoUpdated) {
      logger.error(`${FILE_NAME}: Failed to update repository. Database may not be up-to-date.`);
    }
    
    // Connect to database
    const db = await createDbConnection();
    
    // Initialize database schema
    await initializeDatabase(db);
    
    // Import rules
    await importRules(db);
    
    // Create indexes
    await createIndexes(db);
    
    // Create FTS5 table
    await createFtsTable(db);
    
    // Populate FTS5 table with rule data
    await populateFtsTable(db);
    
    // Close database connection
    db.close((err) => {
      if (err) {
        logger.error(`${FILE_NAME}: Error closing database: ${err.message}`);
      } else {
        logger.info(`${FILE_NAME}: Database connection closed`);
      }
    });
    
    logger.info(`${FILE_NAME}: Database initialization completed successfully`);
    process.exit(0);
  } catch (error) {
    logger.error(`${FILE_NAME}: Database initialization failed: ${error.message}`);
    process.exit(1);
  }
}

// If this script is run directly (not imported)
if (require.main === module) {
  main();
}

module.exports = {
  initializeDatabase,
  importRules,
  createIndexes,
  createFtsTable,
  populateFtsTable
};