/**
 * error_handler.js
 * 
 * Provides standardized error handling for Slack responses
 */
const logger = require('./logger');

/**
 * Handle errors consistently across handlers
 * 
 * @param {Error} error - The error that occurred
 * @param {string} source - Source file/function where error occurred
 * @param {Function} respond - Slack respond function
 * @param {Object} options - Additional options
 * @param {boolean} options.replaceOriginal - Whether to replace original message
 * @param {string} options.responseType - Response type (ephemeral/in_channel)
 * @param {string} options.customMessage - Custom message to display instead of error
 */
const handleError = async (error, source, respond, options = {}) => {
  const { 
    replaceOriginal = false, 
    responseType = 'ephemeral',
    customMessage = null
  } = options;
  
  // Log the error with consistent format
  logger.error(`${source}: ${error.message}`);
  logger.debug(`${source}: Error stack: ${error.stack}`);
  
  // Respond to user with appropriate message
  const displayMessage = customMessage || `An unexpected error occurred: ${error.message}`;
  
  await respond({
    text: displayMessage,
    replace_original: replaceOriginal,
    response_type: responseType
  });
};

module.exports = {
  handleError
};